# -*- coding: utf-8 -*-

# Description: Remark.DocumentTypes package initialization
# Documentation: document_types.txt

# Remark 1.7.2
# Copyright (c) 2009 - 2016
# Kalle Rutanen
# Distributed under the MIT license (see license.txt).

__all__ = [
          	'CodeView_DocumentType',
            'Copy_DocumentType',
          	'CppCodeView_DocumentType',
          	'DirectoryView_DocumentType',
         	'Orphan_DocumentType',
         	'RemarkPage_DocumentType',
          ]
 
