# Description: Version-specific constants
# Documentation: implementation.txt

# Remark 1.7.2
# Copyright (c) 2009 - 2016
# Kalle Rutanen
# Distributed under the MIT license (see license.txt).

def remarkVersion():
    '''
    Returns the version of Remark as a string.
    '''
    return '1.7.2'

